//***********************************************
//*												*
//*           	  Exercise 6-1					*
//*												*
//***********************************************
/* Include Header files  */

#include <p18cxxx.h>
#include <adc.h>
#include <pwm.h>
#include <timers.h>
#include <delays.h>
#include <usart.h>
#include <stdlib.h>
#include <ctype.h>
#include <i2c.h>
#include "P18F_LCD.h"
#include "I2C_EEP_SUBS_24LC32A.h"

	#pragma config	OSC=HS
	#pragma config 	PWRT=OFF
	#pragma config 	BOREN=OFF, BORV=1
	#pragma config 	WDT=OFF
	#pragma config  CCP2MX=PORTC
	#pragma config  STVREN=OFF, LVP=OFF, DEBUG=OFF
	#pragma config  CPD=OFF

/* Declare the Function Prototype */
//void InitializeAD(void);
//void InitializePWM2(void);
//void InitializeTMR2(void);
void InitializePORT(void);
void InitializeUSART(void);
void InitializeINTs(void);
void Initialize_I2C_Master(void);
void isr_high_direct(void);
void isr_high(void);
void Display_Terminal(const rom far char *data);
void Send_CR_LF(void);
 
#define NULL 0x00
#define Min_Langth 4

/* define the LCD_MSG1[] in program memory at 0x1000 */
//***************************************************** 
#pragma romdata My_romdata=0x1000
const rom far char LCD_MSG1[]="Waiting Data Now!";
const rom far char LCD_MSG2[]="-Display EEPROM- ";

const rom far char Disp_Msg1[]="Please Enter your Number :";
const rom far char Disp_Msg2[]="EEPROM WRITE is completed!"; 
const rom far char Disp_Msg3[]="\aError! Please enter again:";
const rom far char Disp_Msg4[]="\aYou must get 4 Digials at least! ";
const rom far char Disp_Msg5[]="Completed to save your Number!";
#pragma romdata

/* define following variable in data memory at Access Bank */
//*********************************************************** 
#pragma udata access My_RAM_2

near unsigned char f;
near unsigned int j;

union					
{
int l;
char i[2];
} near EE_Read_Data;

#pragma udata

unsigned char Rec_Data;
unsigned int EE_Addr;


//***********************************************************
/*														   */
/* 						Main 	Program					   */
/*														   */
//***********************************************************


void main(void)
{

	InitializePORT( );
//	InitializePWM2( );
//	InitializeAD( );		
//	InitializeTMR2( );
	OpenLCD( );
	Initialize_I2C_Master( );
	InitializeINTs( );							
	InitializeUSART( );

	Display_Terminal(Disp_Msg1);					// Send Disp_Msg1 to Hyper-Terminal

 	LCD_Set_Cursor(0,0);						// Put LCD Cursor on Line 1
	putrsLCD(LCD_MSG1);

	Rec_Data=0x00;									// Clear Receive data, it is from RS-232
	EE_Addr=0;										// Set EEPROM start address = 0

	do
	{
		while(Rec_Data==0x00);									// New RS-232 data received ? No, Loop here.
		

			if (isdigit(Rec_Data ))								// Received data is a decimal digit
				{
					f=EEByteWrite_L (0xA0,EE_Addr,Rec_Data);		// Write Receive Data into EEPROM
					f=EEAckPolling(0xA0);						// Polling the Acknowledge
					EE_Addr++;									// Point to next address of EEPROM 
					Rec_Data=0x00;								
					Display_Terminal(Disp_Msg2);				// Send Disp_Msg2 to Hyper-Terminal
				}
				else if (Rec_Data!=0x0D) 						// Press "Enter" key or alphabetic character
					{
						Display_Terminal(Disp_Msg3);			// alphabetic, display error message to Hyper-Terminal
						Rec_Data=0x00;
					}
					else if (EE_Addr<Min_Langth) 				// "Enter" key, need to check how many digit stored
						{
							Display_Terminal(Disp_Msg4);		// send error message to Hyper_Terminal
							Rec_Data=0x00;
						}
		
		
	} while ((EE_Addr<10) && ((Rec_Data!=0x0d)||(EE_Addr<Min_Langth)));  // Fully check

	{															// Write NULL into EEPROM for a string format
		Rec_Data=NULL;
		f=EEByteWrite_L (0xA0,EE_Addr,Rec_Data);
		f=EEAckPolling(0xA0);
		Display_Terminal(Disp_Msg5);
	}

	LCD_Set_Cursor(0,0);										// Put LCD Cursor on Line 1
	putrsLCD(LCD_MSG2);							

	LCD_Set_Cursor(1,0);										// Put LCD Cursor on Line 2
	j=0;

	while(1)													// loop here until read the NULL from EEPROM
	{
		EE_Read_Data.l = EERandomRead_L(0xA0,j);					// Read data from EEPROM

		if (EE_Read_Data.i[0]!=NULL)							// check NULL data?
			{
				WriteDataLCD(EE_Read_Data.i[0]);				// No, display the data to LCD
				j++;											// Point to next address of EEPROM
			}
		else	break;											//Detected NUll, Break out the loop
	}



	while(1);													// End of process, loop
}
//***********************************************
//*   Send a ASCII string to Terminal           *
//***********************************************
void Display_Terminal(const rom far char *data)	
{	
		Send_CR_LF();
		putrsUSART(data);
		Delay10KTCYx(10);
}
void Send_CR_LF(void)
{
		while (BusyUSART());
		WriteUSART('\n');
		while (BusyUSART());
		WriteUSART('\r');
}
//************************************************
//*       #pragma Interrupt Declarations         *
//************************************************
//************************************************
//* Function: isr_high_direct                    *
//*   - Direct execution to the actual           *
//*     high-priority interrupt code.            *
//************************************************
#pragma code isrhighcode = 0x0008

void isr_high_direct(void)	
{  								
     _asm 		//begin in-line assembly
     goto isr_high	//go to isr_high function
     _endasm 		//end in-line assembly
}
#pragma code

//************************************************
//* Function: isr_high(void)                     *     
//*   High priority interrupt will               *
//*   - Received a serial data from RS-232       *
//*     Save the received data to buffer Rec_Data*                  
//************************************************
#pragma interrupt isr_high 

void isr_high(void)
{
	Rec_Data=ReadUSART();				// Get RS-232 data
// 	PORTD=Rec_Data;						// Display to LEDs
}								
#pragma code

//************************************************
//* Function: Initialize Interrupts              *
//*   - Reset All Interrupr                      *
//*   - Enable HIgh Priority Interrupt           *
//*   - Disable Low Priority Interrupt           *
//************************************************
void InitializeINTs(void) 
{
 	PIR1=0; 						//Clear all Interrupt setting & Flag
	PIR2=0;
	INTCON=0;
	INTCON2=0;
	INTCON3=0;
	PIE1=0;
	PIE2=0;

	RCONbits.IPEN=1;     			//Enable priority interrupts

	INTCONbits.GIEL=0;				// Disable Low Priority Interrupt
	INTCONbits.GIEH=1;				// Enalbe high priority Interrupt
}
//************************************************
//* Initial I2C Master Mode with 7 bits Address  *
//* Clock Speed : 100KHz @16MHz					 *
//************************************************
void Initialize_I2C_Master(void)
{
	OpenI2C(MASTER,SLEW_ON);
	SSPADD= 28;
}
//************************************************
//* Function: Initialize USART                   *
//*   - Initialize the RS-232 communication      *
//*   - Disable Interrupr for TxD                *
//*   - Enable Interrupt fro Rxd                 *
//*   - Format: 9600, N, 8, 1                    *
//************************************************
void InitializeUSART(void)
{
	OpenUSART( USART_TX_INT_OFF 		// Set TXSTA Reg. =0b00100100 
			 & USART_RX_INT_ON			// Set RCSTA Reg. =0b10010000 
		 	 & USART_ASYNCH_MODE
			 & USART_EIGHT_BIT
			 & USART_CONT_RX
			 & USART_BRGH_HIGH
			 ,103);				// Set SPBRG=103, Baud Rate = 9600

	PIE1bits.TXIE=0;
	PIR1bits.RCIF=0;
	IPR1bits.RCIP=1;       				// Set Receive of USART are High priority
	PIE1bits.RCIE=1;					// Enable RxD Interrupt
}

//************************************************
//* PIC18F4520 b ADC Peripheral Library k  *
//*  V5 , bϥ OpenADC() ɭnѦ 			 *
//* Function: InitializeAD                       *
//*        - Fosc/32 clock source                *
//*        - Right justified                     *
//*        - 1 analog channels, 0 references     *
//*        - Channel 0                           *
//*        - Disable A/D interrupt               *
//************************************************
void InitializeAD(void)
{
	OpenADC(  ADC_FOSC_32
			& ADC_RIGHT_JUST 
			& ADC_4_TAD,
			  ADC_CH0 
			& ADC_INT_OFF
			& ADC_REF_VDD_VSS,
			  ADC_1ANA); 
}
//************************************************
//* Function: InitializePWM2                     *
//*   - Initialize the CCP2 Module as PWM output *
//*     for controlling motor speed              *
//*     (see p160 MPLAB-CXX Reference Guide)     *
//*        - Set the period to FFh               *
//*        - Make RC1 an output                  *
//*        - Configure CCP2 for PWM              *
//************************************************
void InitializePWM2(void)
{
	TRISCbits.TRISC1=0;	//* setup RC1 for PWM output
	OpenPWM2(0xff); 	//* set period to 0xFF
}


//************************************************
//* Function: InitializeTimer2                   *
//*   - Initialize Timer2 for use with CCP1 PWM  *
//*        - 1:16 Postscale                      *
//*        - 1:4 Prescale                        *
//*        - Enable the timer                    *
//************************************************
void InitializeTMR2(void)
{
	OpenTimer2(T2_PS_1_4 & T2_POST_1_16);
}
//************************************************
//* Function: InitializePORT                     *
//*   - Make PORTD all outputs                   *
//*   - Turn 0ff all the LED's                   *
//************************************************
void InitializePORT(void)
{
	PORTA=0x00;
	TRISA=0b11011011;	// RA2 as LCD-E control, RA5 as CS Control  for SPI	
	PORTD=0x00;
	TRISD=0x00;		// Set PORTD as Output port
}
