
#include <p18cxxx.h>
#include <adc.h>
#include <pwm.h>
#include <timers.h>
#include <delays.h>
#include <usart.h>
#include <stdlib.h>
#include <ctype.h>
#include <i2c.h>
#include "p18F_LCD.h"

 
/* Declare the Function Prototype */
void InitializeAD(void);
void InitializePWM1(void);
void InitializeTMR2(void);
void InitializePORT(void);
void InitializeUSART(void);
void InitializeINTs(void);
void Initialize_I2C_Master(void);
void isr_high_direct(void);
void isr_high(void);
void Display_Terminal(const rom far char *data);
void Send_CR_LF(void);

//************************************************
//* Function: Initialize Interrupts              *
//*   - Reset All Interrupr                      *
//*   - Enable HIgh Priority Interrupt           *
//*   - Disable Low Priority Interrupt           *
//************************************************
void InitializeINTs(void) 
{
 	PIR1=0; 						//Clear all Interrupt setting & Flag
	PIR2=0;
	INTCON=0;
	INTCON2=0;
	INTCON3=0;
	PIE1=0;
	PIE2=0;

	RCONbits.IPEN=1;     			//Enable priority interrupts

	INTCONbits.GIEL=0;				// Disable Low Priority Interrupt
	INTCONbits.GIEH=1;				// Enalbe high priority Interrupt
}
//************************************************
//* Initial I2C Master Mode with 7 bits Address  *
//* Clock Speed : 100KHz @4MHz					 *
//************************************************
void Initialize_I2C_Master(void)
{
	OpenI2C(MASTER,SLEW_ON);
	SSPADD= 9;
}
//************************************************
//* Function: Initialize USART                   *
//*   - Initialize the RS-232 communication      *
//*   - Disable Interrupr for TxD                *
//*   - Enable Interrupt fro Rxd                 *
//*   - Format: 9600, N, 8, 1                    *
//************************************************
void InitializeUSART(void)
{
	OpenUSART( USART_TX_INT_OFF 		// Set TXSTA Reg. =0b00100100 
			 & USART_RX_INT_ON			// Set RCSTA Reg. =0b10010000 
		 	 & USART_ASYNCH_MODE
			 & USART_EIGHT_BIT
			 & USART_CONT_RX
			 & USART_BRGH_HIGH
			 ,103);						// Set SPBRG=103, Baud Rate = 9600 , Fosc=16Mhz

	PIE1bits.TXIE=0;
	PIR1bits.RCIF=0;
	IPR1bits.RCIP=1;       				// Set Receive of USART are High priority
	PIE1bits.RCIE=1;					// Enable RxD Interrupt
}

//************************************************
//* PIC18F4520 b ADC Peripheral Library k  *
//*  V5 , bϥ OpenADC() ɭnѦ 			 *
//* Function: InitializeAD                       *
//*        - Fosc/32 clock source                *
//*        - Right justified                     *
//*        - 1 analog channels, 0 references     *
//*        - Channel 0                           *
//*        - Disable A/D interrupt               *
//************************************************
void InitializeAD(void)
{
	OpenADC(  ADC_FOSC_32
			& ADC_RIGHT_JUST 
			& ADC_4_TAD,
			  ADC_CH0 
			& ADC_INT_OFF
			& ADC_REF_VDD_VSS,
			  ADC_1ANA);
}
//************************************************
//* Function: InitializePWM1                     *
//*   - Initialize the CCP1 Module as PWM output *
//*     for controlling motor speed              *
//*     (see p160 MPLAB-CXX Reference Guide)     *
//*        - Set the period to FFh               *
//*        - Make RC2 an output                  *
//*        - Configure CCP1 for PWM              *
//************************************************
void InitializePWM1(void)
{
	TRISCbits.TRISC2=0;	//* setup RC2 for PWM output
	OpenPWM1(0xff); 	//* set period to 0xFF
}


//************************************************
//* Function: InitializeTimer2                   *
//*   - Initialize Timer2 for use with CCP1 PWM  *
//*        - 1:16 Postscale                      *
//*        - 1:4 Prescale                        *
//*        - Enable the timer                    *
//************************************************
void InitializeTMR2(void)
{
	OpenTimer2(T2_PS_1_4 & T2_POST_1_16);
}
//************************************************
//* Function: InitializePORT                     *
//*   - Make PORTD all outputs                   *
//*   - Turn 0ff all the LED's                   *
//************************************************
void InitializePORT(void)
{
	TRISD=0;
	PORTD=0x00;
}
